@@ -30,7 +30,7 @@ check_formula <- function(formula, varnames, valnames) {
   vars = all.vars(formula)
   vars = vars[!vars %chin% c(".", "...")]
   allvars = c(vars, valnames)
-  if (any(allvars %in% varnames[duplicated(varnames)]))
+  if (any(allvars %chin% varnames[duplicated(varnames)]))
     stop('data.table to cast must have unique column names')
   ans = deparse_formula(as.list(formula)[-1L], varnames, allvars)
 }
@@ -56,7 +56,7 @@ value_vars <- function(value.var, varnames) {
     value.var = list(value.var)
   value.var = lapply(value.var, unique)
   valnames = unique(unlist(value.var))
-  iswrong = which(!valnames %in% varnames)
+  iswrong = which(!valnames %chin% varnames)
   if (length(iswrong))
     stop("value.var values [", paste(value.var[iswrong], collapse=", "), "] are not found in 'data'.")
   value.var
@@ -65,11 +65,12 @@ value_vars <- function(value.var, varnames) {
 aggregate_funs <- function(funs, vals, sep="_", ...) {
   if (is.call(funs) && funs[[1L]] == "eval")
     funs = eval(funs[[2L]], parent.frame(2L), parent.frame(2L))
-  if (is.call(funs) && as.character(funs[[1L]]) %in% c("c", "list"))
+  if (is.call(funs) && as.character(funs[[1L]]) %chin% c("c", "list")) {
     funs = lapply(as.list(funs)[-1L], function(x) {
-      if (is.call(x) && as.character(x[[1L]]) %in% c("c", "list")) as.list(x)[-1L] else x
+      if (is.call(x) && as.character(x[[1L]]) %chin% c("c", "list")) as.list(x)[-1L] else x
     })
-  else funs = list(funs)
+  } else funs = eval(funs, parent.frame(2L), parent.frame(2L))
+  if(is.function(funs)) funs = list(funs) # needed for cases as shown in test#1700.1
   if (length(funs) != length(vals)) {
     if (length(vals) == 1L)
       vals = replicate(length(funs), vals)
@@ -77,8 +78,7 @@ aggregate_funs <- function(funs, vals, sep="_", ...) {
   }
   only_one_fun = length(unlist(funs)) == 1L
   dots = list(...)
-  construct_funs <- function(fun, val) {
-    if (!is.list(fun)) fun = list(fun)
+  construct_funs <- function(fun, nm, val) {
     ans = vector("list", length(fun)*length(val))
     nms = vector("character", length(ans))
     k = 1L
@@ -89,14 +89,20 @@ aggregate_funs <- function(funs, vals, sep="_", ...) {
           expr = c(expr, dots)
         ans[[k]] = as.call(expr)
         # changed order of arguments here, #1153
-        nms[k] = if (only_one_fun) j else
-              paste(j, all.names(i, max.names=1L, functions=TRUE), sep=sep)
+        nms[k] = if (only_one_fun) j else paste(j, nm, sep=sep)
         k = k+1L;
       }
     }
     setattr(ans, 'names', nms)
   }
-  ans = mapply(construct_funs, funs, vals, SIMPLIFY=FALSE)
+  ans = lapply(seq_along(funs), function(i) {
+    nm <- names(funs[i])
+    if (is.null(nm) || !nzchar(nm)) {
+      nm <- all.names(funs[[i]], max.names=1L, functions=TRUE)
+    }
+    if (!length(nm)) nm <- paste0("fun", i)
+    construct_funs(funs[i], nm, vals[[i]])
+  })
   as.call(c(quote(list), unlist(ans)))
 }
 
@@ -121,11 +127,11 @@ dcast.data.table <- function(data, formula, fun.aggregate = NULL, sep = "_", ...
   setattr(lvars, 'names', c("lhs", "rhs"))
   # Have to take care of duplicate names, and provide names for expression columns properly.
   varnames = make.unique(vapply_1c(unlist(lvars), all.vars, max.names=1L), sep=sep)
-  dupidx = which(valnames %in% varnames)
+  dupidx = which(valnames %chin% varnames)
   if (length(dupidx)) {
     dups = valnames[dupidx]
     valnames = tail(make.unique(c(varnames, valnames)), -length(varnames))
-    lvals = lapply(lvals, function(x) { x[x %in% dups] = valnames[dupidx]; x })
+    lvals = lapply(lvals, function(x) { x[x %chin% dups] = valnames[dupidx]; x })
   }
   lhsnames = head(varnames, length(lvars$lhs))
   rhsnames = tail(varnames, -length(lvars$lhs))
@@ -187,7 +193,7 @@ dcast.data.table <- function(data, formula, fun.aggregate = NULL, sep = "_", ...
     lhs = shallow(dat, lhsnames); rhs = shallow(dat, rhsnames); val = shallow(dat, valnames)
     # handle drop=TRUE/FALSE - Update: Logic moved to R, AND faster than previous version. Take that... old me :-).
     if (all(drop)) {
-      map = setDT(lapply(list(lhsnames, rhsnames), function(cols) frankv(dat, cols=cols, ties.method="dense")))
+      map = setDT(lapply(list(lhsnames, rhsnames), function(cols) frankv(dat, cols=cols, ties.method="dense", na.last=FALSE))) # #2202 fix
       maporder = lapply(map, order_)
       mapunique = lapply(seq_along(map), function(i) .Call(CsubsetVector, map[[i]], maporder[[i]]))
       lhs = .Call(CsubsetDT, lhs, maporder[[1L]], seq_along(lhs))
