@@ -1,8 +1,10 @@
-foverlaps <- function(x, y, by.x = if (!is.null(key(x))) key(x) else key(y), by.y = key(y), maxgap=0L, minoverlap=1L, type=c("any", "within", "start", "end", "equal"), mult=c("all", "first", "last"), nomatch=getOption("datatable.nomatch"), which = FALSE,  verbose=getOption("datatable.verbose")) {
+foverlaps <- function(x, y, by.x=if (!is.null(key(x))) key(x) else key(y), by.y=key(y), maxgap=0L, minoverlap=1L, type=c("any", "within", "start", "end", "equal"), mult=c("all", "first", "last"), nomatch=getOption("datatable.nomatch"), which=FALSE, verbose=getOption("datatable.verbose")) {
 
   if (!is.data.table(y) || !is.data.table(x)) stop("y and x must both be data.tables. Use `setDT()` to convert list/data.frames to data.tables by reference or as.data.table() to convert to data.tables by copying.")
   maxgap = as.integer(maxgap); minoverlap = as.integer(minoverlap)
-  which = as.logical(which); nomatch = as.integer(nomatch)
+  which = as.logical(which)
+  if (is.null(nomatch)) nomatch = 0L
+  nomatch = as.integer(nomatch)
   if (!length(maxgap) || length(maxgap) != 1L || is.na(maxgap) || maxgap < 0L)
     stop("maxgap must be a non-negative integer value of length 1")
   if (!length(minoverlap) || length(minoverlap) != 1L || is.na(minoverlap) || minoverlap < 1L)
@@ -10,11 +12,9 @@ foverlaps <- function(x, y, by.x = if (!is.null(key(x))) key(x) else key(y), by.
   if (!length(which) || length(which) != 1L || is.na(which))
     stop("which must be a logical vector of length 1. Either TRUE/FALSE")
   if (!length(nomatch) || length(nomatch) != 1L || (!is.na(nomatch) && nomatch!=0L))
-    stop("nomatch must either be NA or 0, or (ideally) NA_integer_ or 0L")
+    stop("nomatch must either be NA or NULL")
   type = match.arg(type)
   mult = match.arg(mult)
-  if (type == "equal")
-    stop("type = 'equal' is not implemented yet. But note that this is just the same as a normal data.table join y[x, ...], unless you are also interested in setting 'minoverlap / maxgap' arguments. But those arguments are not implemented yet as well.")
   # if (maxgap > 0L || minoverlap > 1L) # for future implementation
   if (maxgap != 0L || minoverlap != 1L)
     stop("maxgap and minoverlap arguments are not yet implemented.")
@@ -29,19 +29,17 @@ foverlaps <- function(x, y, by.x = if (!is.null(key(x))) key(x) else key(y), by.
   }
   if (is.numeric(by.y)) {
     if (any(by.y < 0L) || any(by.y > length(y)))
-      stop("Invalid numeric value for 'by.x'; it should be a vector with values 1 <= by.y <= length(y)")
+      stop("Invalid numeric value for 'by.y'; it should be a vector with values 1 <= by.y <= length(y)")
     by.y = names(y)[by.y]
   }
-  if (!length(by.x) || !is.character(by.x))
-    stop("A non-empty vector of column names is required for by.x")
-  if (!length(by.y) || !is.character(by.y))
-    stop("A non-empty vector of column names is required for by.y")
+  if (!is.character(by.x))
+    stop("A non-empty vector of column names or numbers is required for by.x")
+  if (!is.character(by.y))
+    stop("A non-empty vector of column names or numbers is required for by.y")
   if (!identical(by.y, key(y)[seq_along(by.y)]))
-    stop("The first ", length(by.y), " columns of y's key is not identical to the columns specified in by.y.")
+    stop("The first ", length(by.y), " columns of y's key must be identical to the columns specified in by.y.")
   if (anyNA(chmatch(by.x, names(x))))
     stop("Elements listed in 'by.x' must be valid names in data.table 'x'")
-  if (anyNA(chmatch(by.y, names(y))))
-    stop("Elements listed in 'by.y' must be valid names in data.table 'y'")
   if (anyDuplicated(by.x) || anyDuplicated(by.y))
     stop("Duplicate columns are not allowed in overlap joins. This may change in the future.")
   if (length(by.x) != length(by.y))
@@ -52,17 +50,41 @@ foverlaps <- function(x, y, by.x = if (!is.null(key(x))) key(x) else key(y), by.
     stop("y has some duplicated column name(s): ",paste(unique(names(y)[dup.y]),collapse=","),". Please remove or rename the duplicate(s) and try again.")
   xnames = by.x; xintervals = tail(xnames, 2L)
   ynames = by.y; yintervals = tail(ynames, 2L)
-  if (!storage.mode(x[[xintervals[1L]]]) %chin% c("double", "integer") || !storage.mode(x[[xintervals[2L]]]) %chin% c("double", "integer"))
+  xval1 = x[[xintervals[1L]]]; xval2 = x[[xintervals[2L]]]
+  yval1 = y[[yintervals[1L]]]; yval2 = y[[yintervals[2L]]]
+  if (!storage.mode(xval1) %chin% c("double", "integer") || !storage.mode(xval2) %chin% c("double", "integer") || is.factor(xval1) || is.factor(xval2)) # adding factors to the bunch, #2645
     stop("The last two columns in by.x should correspond to the 'start' and 'end' intervals in data.table 'x' and must be integer/numeric type.")
-  if ( any(x[[xintervals[2L]]] - x[[xintervals[1L]]] < 0L) )
-    stop("All entries in column ", xintervals[1L], " should be <= corresponding entries in column ", xintervals[2L], " in data.table 'x'")
-  if (!storage.mode(y[[yintervals[1L]]]) %chin% c("double", "integer") || !storage.mode(y[[yintervals[2L]]]) %chin% c("double", "integer"))
+  if ( isTRUEorNA(any(xval2 - xval1 < 0L)) ) {
+    # better error messages as suggested by @msummersgill in #3007. Thanks for the code too. Placing this inside so that it only runs if the general condition is satisfied. Should error anyway here.. So doesn't matter even if runs all if-statements; takes about 0.2s for anyNA check on 200 million elements .. acceptable speed for stoppage, I think, at least for now.
+    if ( anyNA(xval1) ) {
+      stop("NA values in data.table 'x' start column: '", xintervals[1L],"'. All rows with NA values in the range columns must be removed for foverlaps() to work.")
+    } else if ( anyNA(xval2) ) {
+      stop("NA values in data.table 'x' end column: '", xintervals[2L],"'. All rows with NA values in the range columns must be removed for foverlaps() to work.")
+    } else stop("All entries in column ", xintervals[1L], " should be <= corresponding entries in column ", xintervals[2L], " in data.table 'x'.")
+  }
+  if (!storage.mode(yval1) %chin% c("double", "integer") || !storage.mode(yval2) %chin% c("double", "integer") || is.factor(yval1) || is.factor(yval2)) # adding factors to the bunch, #2645
     stop("The last two columns in by.y should correspond to the 'start' and 'end' intervals in data.table 'y' and must be integer/numeric type.")
-  if ( any(y[[yintervals[2L]]] - y[[yintervals[1L]]] < 0L) )
-    stop("All entries in column ", yintervals[1L], " should be <= corresponding entries in column ", yintervals[2L], " in data.table 'y'")
-
+  if ( isTRUEorNA(any(yval2 - yval1 < 0L) )) {
+    if ( anyNA(yval1) ) {
+      stop("NA values in data.table 'y' start column: '", yintervals[1L],"'. All rows with NA values in the range columns must be removed for foverlaps() to work.")
+    } else if ( anyNA(yval2) ) {
+      stop("NA values in data.table 'y' end column: '", yintervals[2L],"'. All rows with NA values in the range columns must be removed for foverlaps() to work.")
+    } else stop("All entries in column ", yintervals[1L], " should be <= corresponding entries in column ", yintervals[2L], " in data.table 'y'.")
+  }
+  # POSIXct interval cols error check
+  is.POSIXct <- function(x) inherits(x, "POSIXct")
+  posx_chk <- c(is.POSIXct(xval1), is.POSIXct(xval2), is.POSIXct(yval1), is.POSIXct(yval2))
+  if (any(posx_chk) && !all(posx_chk)) {
+    stop("Some interval cols are of type POSIXct while others are not. Please ensure all interval cols are (or are not) of POSIXct type")
+  }
+  # #1143, mismatched timezone
+  getTZ <- function(x) if (is.null(tz <- attr(x, "tzone"))) "" else tz # "" == NULL AFAICT
+  tzone_chk <- c(getTZ(xval1), getTZ(xval2), getTZ(yval1), getTZ(yval2))
+  if (any(tzone_chk != tzone_chk[1L])) {
+    warning("POSIXct interval cols have mixed timezones. Overlaps are performed on the internal numerical representation of POSIXct objects, therefore printed values may give the impression that values don't overlap but their internal representations will. Please ensure that POSIXct type interval cols have identical 'tzone' attributes to avoid confusion.")
+  }
   ## see NOTES below:
-  yclass = c(class(y[[yintervals[1L]]]), class(y[[yintervals[2L]]]))
+  yclass = c(class(yval1), class(yval2))
   isdouble = FALSE; isposix = FALSE
   if ( any(c("numeric", "POSIXct") %chin% yclass) ) {
     # next representive double > x under the given precision (48,56 or 64-bit in data.table) = x*incr
@@ -70,10 +92,9 @@ foverlaps <- function(x, y, by.x = if (!is.null(key(x))) key(x) else key(y), by.
       bits = floor(log2(.Machine$double.eps))
       2 ^ (bits + (getNumericRounding() * 8L))
     }
-    incr = 1 + dt_eps()
     isdouble = TRUE
     isposix = "POSIXct" %chin% yclass
-  } else incr = 1L # integer or Date class for example
+  }
   ## hopefully all checks are over. Now onto the actual task at hand.
   origx = x; x = shallow(x, by.x)
   origy = y; y = shallow(y, by.y)
@@ -90,15 +111,16 @@ foverlaps <- function(x, y, by.x = if (!is.null(key(x))) key(x) else key(y), by.
     mcall = make_call(mcols, quote(c))
     if (type %chin% c("within", "any")) {
       mcall[[3L]] = substitute(
-        if (isposix) unclass(val)*incr # incr is okay since this won't be negative
+        # datetimes before 1970-01-01 are represented as -ve numerics, #3349
+        if (isposix) unclass(val)*(1 + sign(unclass(val))*dt_eps())
         else if (isdouble) {
           # fix for #1006 - 0.0 occurs in both start and end
           # better fix for 0.0, and other -ves. can't use 'incr'
           # hopefully this doesn't open another can of worms
           (val+dt_eps())*(1 + sign(val)*dt_eps())
         }
-        else val+incr,
-        list(val = mcall[[3L]], incr = incr))
+        else val+1L, # +1L is for integer/IDate/Date class, for examples
+        list(val = mcall[[3L]]))
     }
     make_call(c(icall, pos=mcall), quote(list))
   }
@@ -115,7 +137,7 @@ foverlaps <- function(x, y, by.x = if (!is.null(key(x))) key(x) else key(y), by.
     xx = .shallow(xx, cols, retain.key = FALSE)
     ans = bmerge(xx, ii, seq_along(xx), seq_along(xx), integer(0), mult=mult, ops=rep(1L, length(xx)), integer(0), 1L, verbose=verbose, ...)
     # vecseq part should never run here, but still...
-    if (ans$allLen1) ans$starts else vecseq(ans$starts, ans$lens, NULL)
+    if (ans$allLen1) ans$starts else vecseq(ans$starts, ans$lens, NULL) # nocov
   }
   indices <- function(x, y, intervals, ...) {
     if (type == "start") {
