@@ -42,8 +42,7 @@ frankv <- function(x, cols=seq_along(x), order=1L, na.last=TRUE, ties.method=c("
     order = if (length(order) == 1L) c(rep(order, length(cols)), 1L) else c(order, 1L)
     cols = c(cols, ncol(x))
   }
-  xorder  = forderv(x, by=cols, order=order, sort=TRUE, retGrp=TRUE,
-        na.last=if (identical(na.last, FALSE)) na.last else TRUE)
+  xorder  = forderv(x, by=cols, order=order, sort=TRUE, retGrp=TRUE, na.last=if (isFALSE(na.last)) na.last else TRUE)
   xstart  = attr(xorder, 'starts')
   xsorted = FALSE
   if (!length(xorder)) {
