@@ -1,23 +1,45 @@
-test.data.table <- function(verbose=FALSE, pkg="pkg", silent=FALSE, with.other.packages=FALSE, benchmark=FALSE) {
-  if (exists("test.data.table",.GlobalEnv,inherits=FALSE)) {
+test.data.table <- function(verbose=FALSE, pkg="pkg", silent=FALSE, with.other.packages=FALSE, benchmark=FALSE, script=NULL) {
+  if (exists("test.data.table", .GlobalEnv,inherits=FALSE)) {
     # package developer
     # nocov start
-    if ("package:data.table" %in% search()) stop("data.table package is loaded. Unload or start a fresh R session.")
-    d = if (pkg %in% dir()) file.path(getwd(), pkg) else Sys.getenv("CC_DIR")
-    d = file.path(d, "inst/tests")
+    if ("package:data.table" %chin% search()) stop("data.table package is loaded. Unload or start a fresh R session.")
+    rootdir = if (pkg %chin% dir()) file.path(getwd(), pkg) else Sys.getenv("CC_DIR")
+    subdir = file.path("inst","tests")
     # nocov end
   } else {
     # i) R CMD check and ii) user running test.data.table()
-    d = paste0(getNamespaceInfo("data.table","path"),"/tests")
+    rootdir = getNamespaceInfo("data.table","path")
+    subdir = "tests"
   }
-  # for (fn in dir(d,"*.[rR]$",full=TRUE)) {  # testthat runs those
+  fulldir = file.path(rootdir, subdir)
 
-  stopifnot( !(with.other.packages && benchmark) )
-  fn = if (with.other.packages) "other.Rraw"
-       else if (benchmark) "benchmark.Rraw"
-       else "tests.Rraw"
-  fn = file.path(d, fn)
-  if (!file.exists(fn)) stop(fn," does not exist")
+  if (!is.null(script)) {
+    stopifnot(is.character(script), length(script)==1L, !is.na(script), nzchar(script))
+    if (!identical(basename(script), script)) {
+      subdir = dirname(script)
+      fulldir = normalizePath(subdir, mustWork=FALSE)
+      fn = basename(script)
+    } else {
+      fn = script
+    }
+  } else {
+    stopifnot( !(with.other.packages && benchmark) )
+    fn = if (with.other.packages) "other.Rraw"
+         else if (benchmark) "benchmark.Rraw"
+         else "tests.Rraw"
+  }
+  fn = setNames(file.path(fulldir, fn), file.path(subdir, fn))
+  if (!file.exists(fn)) stop(fn," does not exist")   # nocov
+
+  # From R 3.6.0 onwards, we can check that && and || are using only length-1 logicals (in the test suite)
+  # rather than relying on x && y being equivalent to x[[1L]] && y[[1L]]  silently.
+  orig__R_CHECK_LENGTH_1_LOGIC2_ <- Sys.getenv("_R_CHECK_LENGTH_1_LOGIC2_", unset = NA_character_)
+  Sys.setenv("_R_CHECK_LENGTH_1_LOGIC2_" = TRUE)
+  # This environment variable is restored to its previous state (including not defined) after sourcing test script
+
+  oldRNG = suppressWarnings(RNGversion("3.5.0"))
+  # sample method changed in R 3.6 to remove bias; see #3431 for links and notes
+  # This can be removed (and over 120 tests updated) if and when the oldest R version we test and support is moved to R 3.6
 
   oldverbose = options(datatable.verbose=verbose)
   oldenc = options(encoding="UTF-8")[[1L]]  # just for tests 708-712 on Windows
@@ -25,30 +47,100 @@ test.data.table <- function(verbose=FALSE, pkg="pkg", silent=FALSE, with.other.p
   # oldlocale = Sys.getlocale("LC_CTYPE")
   # Sys.setlocale("LC_CTYPE", "")   # just for CRAN's Mac to get it off C locale (post to r-devel on 16 Jul 2012)
 
-  cat("Running",fn,"\n")
+  cat("Running", fn, "\n")
   env = new.env(parent=.GlobalEnv)
-  assign("testDir", function(x)file.path(d,x), envir=env)
+  assign("testDir", function(x) file.path(fulldir, x), envir=env)
   assign("nfail", 0L, envir=env)
   assign("ntest", 0L, envir=env)
   assign("whichfail", NULL, envir=env)
-  setDTthreads(2) # explicitly limit to 2 so as not to breach CRAN policy (but tests are small so should not use more than 2 anyway)
   assign("started.at", proc.time(), envir=env)
   assign("lasttime", proc.time()[3L], envir=env)  # used by test() to attribute time inbetween tests to the next test
-  assign("timings", data.table( ID = seq_len(3000L), time=0.0, nTest=0L ), envir=env)   # test timings aggregated to integer id
+  assign("timings", data.table( ID = seq_len(9999L), time=0.0, nTest=0L ), envir=env)   # test timings aggregated to integer id
   assign("memtest", as.logical(Sys.getenv("TEST_DATA_TABLE_MEMTEST", "FALSE")), envir=env)
   assign("filename", fn, envir=env)
   assign("inittime", as.integer(Sys.time()), envir=env) # keep measures from various test.data.table runs
   # It doesn't matter that 3000L is far larger than needed for other and benchmark.
   if(isTRUE(silent)){
-    try(sys.source(fn,envir=env), silent=silent)  # nocov
+    try(sys.source(fn, envir=env), silent=silent)  # nocov
   } else {
-    sys.source(fn,envir=env)
+    sys.source(fn, envir=env)
   }
   options(oldverbose)
   options(oldenc)
   # Sys.setlocale("LC_CTYPE", oldlocale)
-  setDTthreads(0)
-  invisible(env$nfail==0)
+  ans = env$nfail==0
+
+  if (is.na(orig__R_CHECK_LENGTH_1_LOGIC2_)) {
+    Sys.unsetenv("_R_CHECK_LENGTH_1_LOGIC2_")
+  } else {
+    Sys.setenv("_R_CHECK_LENGTH_1_LOGIC2_" = orig__R_CHECK_LENGTH_1_LOGIC2_)   # nocov
+  }
+
+  suppressWarnings(do.call("RNGkind",as.list(oldRNG)))
+  # suppressWarning again in the unlikely event that user selected sample='Rounding' themselves before calling test.data.table()
+
+  timings = get("timings", envir=env)
+  ntest = get("ntest", envir=env)
+  nfail = get("nfail", envir=env)
+  started.at = get("started.at", envir=env)
+  whichfail = get("whichfail", envir=env)
+
+  # Summary. This code originally in tests.Rraw and moved up here in #3307
+  # One big long line because CRAN checks output last 13 lines. One long line counts as one out of 13.
+  plat = paste0("endian==", .Platform$endian,
+                ", sizeof(long double)==", .Machine$sizeof.longdouble,
+                ", sizeof(pointer)==", .Machine$sizeof.pointer,
+                ", TZ=", suppressWarnings(Sys.timezone()),
+                ", locale='", Sys.getlocale(), "'",
+                ", l10n_info()='", paste0(names(l10n_info()), "=", l10n_info(), collapse="; "), "'",
+                ", getDTthreads()='", paste0(capture.output(invisible(getDTthreads(verbose=TRUE))), collapse="; "), "'")
+  DT = head(timings[-1L][order(-time)],10)   # exclude id 1 as in dev that includes JIT
+  if ((x<-sum(timings[["nTest"]])) != ntest) {
+    warning("Timings count mismatch:",x,"vs",ntest)  # nocov
+  }
+  cat("\n10 longest running tests took ", as.integer(tt<-DT[, sum(time)]), "s (", as.integer(100*tt/(ss<-timings[,sum(time)])), "% of ", as.integer(ss), "s)\n", sep="")
+  print(DT, class=FALSE)
+
+  ## this chunk requires to include new suggested deps: graphics, grDevices
+  #memtest.plot = function(.inittime) {
+  #  if (!all(requireNamespace(c("graphics","grDevices"), quietly=TRUE))) return(invisible())
+  #  inittime=PS_rss=GC_used=GC_max_used=NULL
+  #  m = fread("memtest.csv")[inittime==.inittime]
+  #  if (nrow(m)) {
+  #    ps_na = all(is.na(m[["PS_rss"]])) # OS with no 'ps -o rss R' support
+  #    grDevices::png("memtest.png")
+  #    p = graphics::par(mfrow=c(if (ps_na) 2 else 3, 2))
+  #    if (!ps_na) {
+  #      m[, graphics::plot(test, PS_rss, pch=18, xlab="test num", ylab="mem MB", main="ps -o rss R")]
+  #      m[, graphics::plot(timestamp, PS_rss, type="l", xlab="timestamp", ylab="mem MB", main="ps -o rss R")]
+  #    }
+  #    m[, graphics::plot(test, GC_used, pch=18, xlab="test num", ylab="mem MB", main="gc used")]
+  #    m[, graphics::plot(timestamp, GC_used, type="l", xlab="timestamp", ylab="mem MB", main="gc used")]
+  #    m[, graphics::plot(test, GC_max_used, pch=18, xlab="test num", ylab="mem MB", main="gc max used")]
+  #    m[, graphics::plot(timestamp, GC_max_used, type="l", xlab="timestamp", ylab="mem MB", main="gc max used")]
+  #    graphics::par(p)
+  #    grDevices::dev.off()
+  #  } else {
+  #    warning("test.data.table runs with memory testing but did not collect any memory statistics.")
+  #  }
+  #}
+  #if (memtest<-get("memtest", envir=env)) memtest.plot(get("inittime", envir=env))
+
+  if (nfail > 0) {
+    # nocov start
+    if (nfail>1) {s1="s";s2="s: "} else {s1="";s2=" "}
+    cat("\r")
+    stop(nfail," error",s1," out of ",ntest," in ",timetaken(started.at)," on ",date(),". [",plat,"].",
+         " Search ",names(fn)," for test number",s2,paste(whichfail,collapse=", "),".")
+    # important to stop() here, so that 'R CMD check' fails
+    # nocov end
+  }
+  cat(plat,"\n\nAll ",ntest," tests in ",names(fn)," completed ok in ",timetaken(started.at)," on ",date(),"\n",sep="")
+  # date() is included so we can tell exactly when these tests ran on CRAN. Sometimes a CRAN log can show error but that can be just
+  # stale due to not updating yet since a fix in R-devel, for example.
+
+  #attr(ans, "details") <- env
+  invisible(ans)
 }
 
 # nocov start
@@ -134,7 +226,8 @@ test <- function(num,x,y=TRUE,error=NULL,warning=NULL,output=NULL,message=NULL)
     filename = NA_character_ # nocov
   }
 
-  if (!missing(error) && !missing(y)) stop("Test ",num," is invalid: when error= is provided it does not make sense to pass y as well")
+  if (!missing(error) && !missing(y))
+    stop("Test ",num," is invalid: when error= is provided it does not make sense to pass y as well")  # nocov
 
   string_match = function(x, y) {
     length(grep(x,y,fixed=TRUE)) ||                    # try treating x as literal first; useful for most messages containing ()[]+ characters
@@ -212,7 +305,7 @@ test <- function(num,x,y=TRUE,error=NULL,warning=NULL,output=NULL,message=NULL)
   }
   if (!fail && !length(error) && (!length(output) || !missing(y))) {   # TODO test y when output=, too
     y = try(y,TRUE)
-    if (identical(x,y)) return(invisible())
+    if (identical(x,y)) return(invisible(TRUE))
     all.equal.result = TRUE
     if (is.data.table(x) && is.data.table(y)) {
       if (!selfrefok(x) || !selfrefok(y)) {
@@ -230,12 +323,12 @@ test <- function(num,x,y=TRUE,error=NULL,warning=NULL,output=NULL,message=NULL)
         setattr(yc,"row.names",NULL)
         setattr(xc,"index",NULL)   # too onerous to create test RHS with the correct index as well, just check result
         setattr(yc,"index",NULL)
-        if (identical(xc,yc) && identical(key(x),key(y))) return(invisible())  # check key on original x and y because := above might have cleared it on xc or yc
+        if (identical(xc,yc) && identical(key(x),key(y))) return(invisible(TRUE))  # check key on original x and y because := above might have cleared it on xc or yc
         if (isTRUE(all.equal.result<-all.equal(xc,yc)) && identical(key(x),key(y)) &&
-          identical(vapply_1c(xc,typeof), vapply_1c(yc,typeof))) return(invisible())
+          identical(vapply_1c(xc,typeof), vapply_1c(yc,typeof))) return(invisible(TRUE))
       }
     }
-    if (is.atomic(x) && is.atomic(y) && isTRUE(all.equal.result<-all.equal(x,y,check.names=!isTRUE(y))) && typeof(x)==typeof(y)) return(invisible())
+    if (is.atomic(x) && is.atomic(y) && isTRUE(all.equal.result<-all.equal(x,y,check.names=!isTRUE(y))) && typeof(x)==typeof(y)) return(invisible(TRUE))
     # For test 617 on r-prerel-solaris-sparc on 7 Mar 2013
     # nocov start
     if (!fail) {
@@ -255,6 +348,6 @@ test <- function(num,x,y=TRUE,error=NULL,warning=NULL,output=NULL,message=NULL)
     assign("whichfail", c(whichfail, num), parent.frame(), inherits=TRUE)
     # nocov end
   }
-  invisible()
+  invisible(!fail)
 }
 
