@@ -1,12 +1,15 @@
 #include "data.table.h"
 #include <Rdefines.h>
 #include <time.h>
+// #include <signal.h> // the debugging machinery + breakpoint aidee
 
+// TODO: rewrite/simplify logic -- took me ages to understand what I wrote!!
+// TODO: benchmark and parallelise slow regions
 // TODO: implement 'lookup' for 'gaps' and 'overlaps' arguments
 SEXP lookup(SEXP ux, SEXP xlen, SEXP indices, SEXP gaps, SEXP overlaps, SEXP multArg, SEXP typeArg, SEXP verbose) {
 
   SEXP vv, tt, lookup, type_lookup;
-  R_len_t i,j,k,*idx,*len1,*len2,xrows=INTEGER(xlen)[0],uxrows=LENGTH(VECTOR_ELT(ux, 0)),uxcols=LENGTH(ux);
+  R_len_t i,j,k,*idx,*count,*type_count,xrows=INTEGER(xlen)[0],uxrows=LENGTH(VECTOR_ELT(ux, 0)),uxcols=LENGTH(ux);
   int *from = (int *)INTEGER(VECTOR_ELT(indices, 0));
   int *to   = (int *)INTEGER(VECTOR_ELT(indices, 1));
   clock_t pass1, pass2, pass3, start;
@@ -22,25 +25,34 @@ SEXP lookup(SEXP ux, SEXP xlen, SEXP indices, SEXP gaps, SEXP overlaps, SEXP mul
   else if (!strcmp(CHAR(STRING_ELT(typeArg, 0)), "within")) type = WITHIN;
   else if (!strcmp(CHAR(STRING_ELT(typeArg, 0)), "start")) type = START;
   else if (!strcmp(CHAR(STRING_ELT(typeArg, 0)), "end")) type = END;
-  // TODO : re-cover when implemented
-  else if (!strcmp(CHAR(STRING_ELT(typeArg, 0)), "equal")) type = EQUAL; // # nocov
+  else if (!strcmp(CHAR(STRING_ELT(typeArg, 0)), "equal")) type = EQUAL;
   else error("Internal error: invalid value for 'type'; this should have been caught before. please report to data.table issue tracker"); // # nocov
 
   // For reference: uxcols-1 = type_count, uxcols-2 = count, uxcols-3 = type_lookup, uxcols-4 = lookup
   // first pass: calculate lengths first
   start = clock();
-  len1 = (int *)INTEGER(VECTOR_ELT(ux, uxcols-2));
-  len2 = (int *)INTEGER(VECTOR_ELT(ux, uxcols-1));
+  count = (int *)INTEGER(VECTOR_ELT(ux, uxcols-2));
+  type_count = (int *)INTEGER(VECTOR_ELT(ux, uxcols-1));
   switch (mult) {
   case FIRST:
-    for (i=0; i<xrows; i++) {
-      for (j=from[i]; j<=to[i]; j++) {
-        len1[j-1]++;
+    switch(type) {
+    case EQUAL:
+      for (i=0; i<xrows; i++) {
+        count[from[i]-1]++; count[to[i]-1]++;
+        type_count[from[i]-1]++; type_count[to[i]-1]++;
       }
-    }
-    if (type != WITHIN) {
-      for (i=0; i<uxrows; i++)                      // TODO: this allocation can be avoided if we take care of FIRST/LAST accordingly in 'overlaps'
-        if (len1[i]) len2[i] = 1;
+      break;
+    case START: case END: case ANY: case WITHIN:
+      for (i=0; i<xrows; i++) {
+        for (j=from[i]; j<=to[i]; j++) {
+          count[j-1]++;
+        }
+      }
+      if (type != WITHIN) {
+        for (i=0; i<uxrows; i++)                      // TODO: this allocation can be avoided if we take care of FIRST/LAST accordingly in 'overlaps'
+          if (count[i]) type_count[i] = 1;
+      }
+      break;
     }
     break;
 
@@ -49,20 +61,26 @@ SEXP lookup(SEXP ux, SEXP xlen, SEXP indices, SEXP gaps, SEXP overlaps, SEXP mul
     case ANY:
       for (i=0; i<xrows; i++) {
         for (j=from[i]; j<=to[i]; j++) {
-          len1[j-1]++;
-          if (from[i]==j && !len2[j-1]) len2[j-1]++;
+          count[j-1]++;
+          if (from[i]==j && !type_count[j-1]) type_count[j-1]++;
         }
       }
       break;
-    case START: case END: case EQUAL: case WITHIN:
+    case EQUAL:
+      for (i=0; i<xrows; i++) {
+        count[from[i]-1]++; count[to[i]-1]++;
+        type_count[from[i]-1]++; type_count[to[i]-1]++;
+      }
+      break;
+    case START: case END: case WITHIN:
       for (i=0; i<xrows; i++) {
         for (j=from[i]; j<=to[i]; j++) {
-          len1[j-1]++;
+          count[j-1]++;
         }
       }
       if (type != WITHIN) {
         for (i=0; i<uxrows; i++)              // TODO: this allocation can be avoided if we take care of FIRST/LAST accordingly in 'overlaps'
-          if (len1[i]) len2[i] = 1;
+          if (count[i]) type_count[i] = 1;
       }
       break;
     }
@@ -73,32 +91,29 @@ SEXP lookup(SEXP ux, SEXP xlen, SEXP indices, SEXP gaps, SEXP overlaps, SEXP mul
     case START: case END:
       for (i=0; i<xrows; i++) {
         for (j=from[i]; j<=to[i]; j++) {
-          len1[j-1]++; len2[j-1]++;       // alternatively, we could simply do with len2=len1 ?
+          count[j-1]++; type_count[j-1]++;       // alternatively, we could simply do with type_count=count ?
         }
       }
       break;
-    // TODO : re-cover when implemented
-    // # nocov start
     case EQUAL:
       for (i=0; i<xrows; i++) {
-        len1[from[i]-1]++; len1[to[i]-1]++;
-        len2[from[i]-1]++; len2[to[i]-1]++;
+        count[from[i]-1]++; count[to[i]-1]++;
+        type_count[from[i]-1]++; type_count[to[i]-1]++;
       }
       break;
-    // # nocov end
     case ANY :
       for (i=0; i<xrows; i++) {
         k = from[i];
         for (j=from[i]; j<=to[i]; j++) {
-          len1[j-1]++;
-          if (k==j) len2[j-1]++;
+          count[j-1]++;
+          if (k==j) type_count[j-1]++;
         }
       }
       break;
     case WITHIN :
       for (i=0; i<xrows; i++) {
         for (j=from[i]; j<=to[i]; j++) {
-          len1[j-1]++;
+          count[j-1]++;
         }
       }
       break;
@@ -113,9 +128,9 @@ SEXP lookup(SEXP ux, SEXP xlen, SEXP indices, SEXP gaps, SEXP overlaps, SEXP mul
   lookup = VECTOR_ELT(ux, uxcols-4);
   type_lookup = VECTOR_ELT(ux, uxcols-3);
   for (i=0; i<uxrows; i++) {
-    SET_VECTOR_ELT(lookup, i, vv=allocVector(INTSXP, len1[i]));
+    SET_VECTOR_ELT(lookup, i, vv=allocVector(INTSXP, count[i]));
     if (type != WITHIN) {
-      SET_VECTOR_ELT(type_lookup, i, vv=allocVector(INTSXP, len2[i]));
+      SET_VECTOR_ELT(type_lookup, i, vv=allocVector(INTSXP, type_count[i]));
     }
   }
   pass2 = clock() - start;
@@ -133,15 +148,12 @@ SEXP lookup(SEXP ux, SEXP xlen, SEXP indices, SEXP gaps, SEXP overlaps, SEXP mul
       }
     }
     break;
-  // TODO : re-cover when implemented
-  // # nocov start
   case EQUAL:
     for (i=0; i<xrows; i++) {
       INTEGER(VECTOR_ELT(lookup, from[i]-1))[idx[from[i]-1]++] = i+1;
       INTEGER(VECTOR_ELT(lookup, to[i]-1))[idx[to[i]-1]++] = i+1;
     }
     break;
-  // # nocov end
   }
   Free(idx);
   // generate type_lookup
@@ -149,7 +161,7 @@ SEXP lookup(SEXP ux, SEXP xlen, SEXP indices, SEXP gaps, SEXP overlaps, SEXP mul
     switch (mult) {
     case FIRST :
       for (i=0; i<uxrows; i++) {
-        if (!len1[i]) continue;
+        if (!count[i]) continue;
         vv = VECTOR_ELT(lookup, i);
         tt = VECTOR_ELT(type_lookup, i);
         if (length(tt) && length(vv)) {            // length check added by Matt to avoid SEGV in #2767
@@ -160,11 +172,11 @@ SEXP lookup(SEXP ux, SEXP xlen, SEXP indices, SEXP gaps, SEXP overlaps, SEXP mul
 
     case LAST :
       for (i=0; i<uxrows; i++) {
-        if (!len1[i]) continue;
+        if (!count[i]) continue;
         vv = VECTOR_ELT(lookup, i);
         tt = VECTOR_ELT(type_lookup, i);
-        if (length(tt) && length(vv)>=len1[i]) {   // length check added by Matt to avoid SEGV in #2767
-          INTEGER(tt)[0] = INTEGER(vv)[len1[i]-1];
+        if (length(tt) && length(vv)>=count[i]) {   // length check added by Matt to avoid SEGV in #2767
+          INTEGER(tt)[0] = INTEGER(vv)[count[i]-1];
         }
       }
 
@@ -180,7 +192,7 @@ SEXP lookup(SEXP ux, SEXP xlen, SEXP indices, SEXP gaps, SEXP overlaps, SEXP mul
           vv = VECTOR_ELT(lookup, i);
           tt = VECTOR_ELT(type_lookup, i);
           k=0;
-          for (j=len1[i]-len2[i]; j<len1[i]; j++)
+          for (j=count[i]-type_count[i]; j<count[i]; j++)
             INTEGER(tt)[k++] = INTEGER(vv)[j];
         }
         break;
@@ -189,7 +201,7 @@ SEXP lookup(SEXP ux, SEXP xlen, SEXP indices, SEXP gaps, SEXP overlaps, SEXP mul
         // for (i=0; i<uxrows; i++) {
         //     vv = VECTOR_ELT(lookup, i);
         //     tt = VECTOR_ELT(type_lookup, i);
-        //     for (j=0; j<len2[i]; j++)
+        //     for (j=0; j<type_count[i]; j++)
         //         INTEGER(tt)[j] = INTEGER(vv)[j];
         // }
         break;
@@ -209,14 +221,15 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
   int nomatch = INTEGER(nomatchArg)[0], totlen=0, len, thislen, wlen=0;
   int *from   = (int *)INTEGER(VECTOR_ELT(imatches, 0));
   int *to     = (int *)INTEGER(VECTOR_ELT(imatches, 1));
-  int *len1   = (int *)INTEGER(VECTOR_ELT(ux, uxcols-2));
-  int *len2   = (int *)INTEGER(VECTOR_ELT(ux, uxcols-1));
+  int *count   = (int *)INTEGER(VECTOR_ELT(ux, uxcols-2));
+  int *type_count   = (int *)INTEGER(VECTOR_ELT(ux, uxcols-1));
   SEXP lookup = VECTOR_ELT(ux, uxcols-4);
   SEXP type_lookup = VECTOR_ELT(ux, uxcols-3);
   SEXP ans, f1__, f2__, tmp1, tmp2;
   clock_t end1, end2, start;
   enum {ALL, FIRST, LAST} mult = ALL;
   enum {ANY, WITHIN, START, END, EQUAL} type = ANY;
+  // raise(SIGINT);
 
   if (!strcmp(CHAR(STRING_ELT(multArg, 0)), "all"))  mult = ALL;
   else if (!strcmp(CHAR(STRING_ELT(multArg, 0)), "first")) mult = FIRST;
@@ -238,25 +251,23 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
     switch (type) {
     case START: case END:
       for (i=0; i<rows; i++)
-        totlen += (from[i] > 0 && len2[from[i]-1]) ? len2[from[i]-1] : 1;
+        totlen += (from[i] > 0 && type_count[from[i]-1]) ? type_count[from[i]-1] : 1;
       break;
 
-    // TODO : re-cover when implemented
-    // # nocov start
     case EQUAL:
       for (i=0; i<rows; i++) {
         len = totlen; wlen=0, j=0, m=0;
         k = (from[i]>0) ? from[i] : 1;
         if (k == to[i]) {
-          wlen = len1[k-1];
+          wlen = count[k-1];
         } else if (k < to[i]) {
           tmp1 = VECTOR_ELT(lookup, k-1);
           tmp2 = VECTOR_ELT(type_lookup, to[i]-1);
-          while (j<len1[k-1] && m<len2[to[i]-1]) {
+          while (j<count[k-1] && m<type_count[to[i]-1]) {
             if ( INTEGER(tmp1)[j] == INTEGER(tmp2)[m] ) {
               ++wlen; ++j; ++m;
             } else if ( INTEGER(tmp1)[j] > INTEGER(tmp2)[m] ) {
-              break;
+              ++m;
             } else ++j;
           }
         }
@@ -265,7 +276,6 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
           ++totlen;
       }
       break;
-    // # nocov end
 
     case ANY:
       for (i=0; i<rows; i++) {
@@ -273,9 +283,9 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
         // k = (from[i] > 0) ? from[i] : 1;
         k = from[i];
         if (k<=to[i])
-          totlen += len1[k-1];
+          totlen += count[k-1];
         for (j=k+1; j<=to[i]; j++)
-          totlen += len2[j-1];
+          totlen += type_count[j-1];
         if (len == totlen)
           ++totlen;
       }
@@ -287,11 +297,11 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
         k = from[i];
         if (k > 0) {
           if (k == to[i]) {
-            totlen += len1[k-1];
+            totlen += count[k-1];
           } else if (k < to[i]) {
             tmp1 = VECTOR_ELT(lookup, k-1);
             tmp2 = VECTOR_ELT(lookup, to[i]-1);
-            while (j<len1[k-1] && m<len1[to[i]-1]) {
+            while (j<count[k-1] && m<count[to[i]-1]) {
               if ( INTEGER(tmp1)[j] == INTEGER(tmp2)[m] ) {
                 ++totlen; ++j; ++m;
               } else if ( INTEGER(tmp1)[j] > INTEGER(tmp2)[m] ) {
@@ -330,7 +340,7 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
         if (from[i] > 0) {
           k = from[i];
           tmp2 = VECTOR_ELT(type_lookup, k-1);
-          for (j=0; j<len2[k-1]; j++) {
+          for (j=0; j<type_count[k-1]; j++) {
             INTEGER(f1__)[thislen] = i+1;
             INTEGER(f2__)[thislen] = INTEGER(tmp2)[j];
             ++thislen;
@@ -344,8 +354,6 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
       }
       break;
 
-    // TODO : re-cover when implemented
-    // # nocov start
     case EQUAL :
       for (i=0; i<rows; i++) {
         len = thislen;
@@ -354,7 +362,7 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
           if (k == to[i]) {
             tmp1 = VECTOR_ELT(lookup, k-1);
             tmp2 = VECTOR_ELT(type_lookup, to[i]-1);
-            for (j=0; j<len1[k-1]; j++) {
+            for (j=0; j<count[k-1]; j++) {
               INTEGER(f1__)[thislen] = i+1;
               INTEGER(f2__)[thislen] = INTEGER(tmp1)[j];
               ++thislen;
@@ -363,7 +371,7 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
             j=0; m=0;
             tmp1 = VECTOR_ELT(lookup, k-1);
             tmp2 = VECTOR_ELT(type_lookup, to[i]-1);
-            while (j<len1[k-1] && m<len2[to[i]-1]) {
+            while (j<count[k-1] && m<type_count[to[i]-1]) {
               if ( INTEGER(tmp1)[j] == INTEGER(tmp2)[m] ) {
                 INTEGER(f1__)[thislen] = i+1;
                 INTEGER(f2__)[thislen] = INTEGER(tmp1)[j];
@@ -381,7 +389,6 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
          }
       }
       break;
-    // # nocov end
 
     case ANY :
       for (i=0; i<rows; i++) {
@@ -390,7 +397,7 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
         k = from[i];
         if (k<=to[i]) {
           tmp1 = VECTOR_ELT(lookup, k-1);
-          for (m=0; m<len1[k-1]; m++) {
+          for (m=0; m<count[k-1]; m++) {
             INTEGER(f1__)[thislen] = i+1;
             INTEGER(f2__)[thislen] = INTEGER(tmp1)[m];
             ++thislen;
@@ -398,7 +405,7 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
         }
         for (j=k+1; j<=to[i]; j++) {
           tmp2 = VECTOR_ELT(type_lookup, j-1);
-          for (m=0; m<len2[j-1]; m++) {
+          for (m=0; m<type_count[j-1]; m++) {
             INTEGER(f1__)[thislen] = i+1;
             INTEGER(f2__)[thislen] = INTEGER(tmp2)[m];
             ++thislen;
@@ -420,7 +427,7 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
         if (k > 0) {
           if (k == to[i]) {
             tmp1 = VECTOR_ELT(lookup, k-1);
-            for (j=0; j<len1[k-1]; j++) {
+            for (j=0; j<count[k-1]; j++) {
               INTEGER(f1__)[thislen] = i+1;
               INTEGER(f2__)[thislen] = INTEGER(tmp1)[j];
               ++thislen;
@@ -429,7 +436,7 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
             j=0; m=0;
             tmp1 = VECTOR_ELT(lookup, k-1);
             tmp2 = VECTOR_ELT(lookup, to[i]-1);
-            while (j<len1[k-1] && m<len1[to[i]-1]) {
+            while (j<count[k-1] && m<count[to[i]-1]) {
               if ( INTEGER(tmp1)[j] == INTEGER(tmp2)[m] ) {
                 INTEGER(f1__)[thislen] = i+1;
                 INTEGER(f2__)[thislen] = INTEGER(tmp1)[j];
@@ -457,7 +464,7 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
         len = thislen;
         INTEGER(f1__)[thislen] = i+1;
         k = (from[i]>0) ? from[i] : 1;
-        if (k <= to[i]) { // len1[k-1] is equal to len2[k-1] and will always be >0, so no length check necessary.
+        if (k <= to[i]) { // count[k-1] is equal to type_count[k-1] and will always be >0, so no length check necessary.
           tmp1 = VECTOR_ELT(lookup, k-1);
           INTEGER(f2__)[thislen] = INTEGER(tmp1)[0];
           ++thislen;
@@ -469,8 +476,6 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
       }
       break;
 
-    // TODO : re-cover when implemented
-    // # nocov start
     case EQUAL :
       for (i=0; i<rows; i++) {
         len = thislen;
@@ -485,7 +490,7 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
             j=0; m=0;
             tmp1 = VECTOR_ELT(lookup, k-1);
             tmp2 = VECTOR_ELT(type_lookup, to[i]-1);
-            while (j<len1[k-1] && m<len2[to[i]-1]) {
+            while (j<count[k-1] && m<type_count[to[i]-1]) {
               if ( INTEGER(tmp1)[j] == INTEGER(tmp2)[m] ) {
                 INTEGER(f2__)[thislen] = INTEGER(tmp1)[j];
                  ++thislen; ++j; ++m;
@@ -502,7 +507,6 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
          }
       }
       break;
-    // # nocov end
 
     case ANY:
       for (i=0; i<rows; i++) {
@@ -511,7 +515,7 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
         // k = (from[i]>0) ? from[i] : 1;
         k = from[i];
         for (j=k; j<=to[i]; j++) {
-          if (len2[j-1]) {
+          if (type_count[j-1]) {
             tmp2 = VECTOR_ELT(type_lookup, j-1);
             INTEGER(f2__)[thislen] = INTEGER(tmp2)[0];
             ++thislen;
@@ -531,7 +535,7 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
         INTEGER(f1__)[thislen] = i+1;
         k = from[i];
         if (k > 0) {
-          if (k == to[i] && len1[k-1]) {
+          if (k == to[i] && count[k-1]) {
             tmp1 = VECTOR_ELT(lookup, k-1);
             INTEGER(f2__)[thislen] = INTEGER(tmp1)[0];
             ++thislen;
@@ -539,7 +543,7 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
             j=0; m=0;
             tmp1 = VECTOR_ELT(lookup, k-1);
             tmp2 = VECTOR_ELT(lookup, to[i]-1);
-            while (j<len1[k-1] && m<len1[to[i]-1]) {
+            while (j<count[k-1] && m<count[to[i]-1]) {
               if ( INTEGER(tmp1)[j] == INTEGER(tmp2)[m] ) {
                 INTEGER(f2__)[thislen] = INTEGER(tmp1)[j];
                  ++thislen; ++j; ++m;
@@ -566,9 +570,9 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
         len = thislen;
         INTEGER(f1__)[thislen] = i+1;
         k = (from[i]>0) ? from[i] : 1;
-        if (k <= to[i]) { // len1[k-1] is equal to len2[k-1] and will always be >0, so no length check necessary.
+        if (k <= to[i]) { // count[k-1] is equal to type_count[k-1] and will always be >0, so no length check necessary.
           tmp1 = VECTOR_ELT(lookup, k-1);
-          INTEGER(f2__)[thislen] = INTEGER(tmp1)[len1[k-1]-1];
+          INTEGER(f2__)[thislen] = INTEGER(tmp1)[count[k-1]-1];
           ++thislen;
         }
         if (len == thislen) {
@@ -578,9 +582,15 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
       }
       break;
 
-    // TODO : re-cover when implemented
-    // # nocov start
     case EQUAL :
+      // Debugging reference for future-me
+      // R -d lldb
+      // run -f file.R
+      // breakpoint set -f ijoin.c -l 591
+      // c (hit enter to break at line 591)
+      // n (next line)
+      // p val # for native C objects
+      // call Rf_PrintValue(val) # for SEXP objects, to print whole vector/vals
       for (i=0; i<rows; i++) {
         len = thislen;
         INTEGER(f1__)[thislen] = i+1;
@@ -588,12 +598,12 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
           k = from[i];
           if (k == to[i]) {
             tmp1 = VECTOR_ELT(lookup, k-1);
-            INTEGER(f2__)[thislen] = INTEGER(tmp1)[len1[k-1]-1];
+            INTEGER(f2__)[thislen] = INTEGER(tmp1)[count[k-1]-1];
             ++thislen;
           } else if (k < to[i]) {
             tmp1 = VECTOR_ELT(lookup, k-1);
             tmp2 = VECTOR_ELT(type_lookup, to[i]-1);
-            j=len1[k-1]-1; m=len2[k-1]-1;
+            j=count[k-1]-1; m=type_count[to[i]-1]-1; // bug fix, k=from[i] but should be to[i]
             while (j>=0 && m>=0) {
               if ( INTEGER(tmp1)[j] == INTEGER(tmp2)[m] ) {
                 INTEGER(f2__)[thislen] = INTEGER(tmp1)[j];
@@ -611,7 +621,6 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
          }
       }
       break;
-    // # nocov end
 
       // OLD logic for 'any,last' which had to check for maximum for each 'i'. Better logic below.
       // for 'first' we need to just get the minimum of first non-zero-length element, but not the same case for 'last'.
@@ -624,9 +633,9 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
       //     // k = (from[i]>0) ? from[i] : 1;
       //     k = from[i];
       //     for (j=k; j<=to[i]; j++) {
-      //         if (len2[j-1]) {
+      //         if (type_count[j-1]) {
       //             tmp2 = VECTOR_ELT(type_lookup, j-1);
-      //             INTEGER(f2__)[thislen] = (INTEGER(f2__)[thislen] < INTEGER(tmp2)[len2[j-1]-1]) ? INTEGER(tmp2)[len2[j-1]-1] : INTEGER(f2__)[thislen];
+      //             INTEGER(f2__)[thislen] = (INTEGER(f2__)[thislen] < INTEGER(tmp2)[type_count[j-1]-1]) ? INTEGER(tmp2)[type_count[j-1]-1] : INTEGER(f2__)[thislen];
       //         }
       //     }
       //     if (INTEGER(f2__)[thislen] == 0)
@@ -642,21 +651,21 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
         // k = (from[i]>0) ? from[i] : 1;
         k = from[i];
         if (k <= to[i]) {
-          if (k==to[i] && len1[k-1]) {
+          if (k==to[i] && count[k-1]) {
             tmp1 = VECTOR_ELT(lookup, k-1);
-            INTEGER(f2__)[thislen] = INTEGER(tmp1)[len1[k-1]-1];
+            INTEGER(f2__)[thislen] = INTEGER(tmp1)[count[k-1]-1];
             ++thislen;
           } else {
             for (j=to[i]; j>k; j--) {
-              if (len2[j-1]) {
+              if (type_count[j-1]) {
                 tmp2 = VECTOR_ELT(type_lookup, j-1);
                 INTEGER(f2__)[thislen] = INTEGER(tmp2)[0]; // tmp2 will be length 1
                 ++thislen; break;
               }
             }
-            if (len == thislen && len1[k-1]) {
+            if (len == thislen && count[k-1]) {
               tmp1 = VECTOR_ELT(lookup, k-1);
-              INTEGER(f2__)[thislen] = INTEGER(tmp1)[len1[k-1]-1];
+              INTEGER(f2__)[thislen] = INTEGER(tmp1)[count[k-1]-1];
               ++thislen;
             }
           }
@@ -674,14 +683,14 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
         INTEGER(f1__)[thislen] = i+1;
         k = from[i];
         if (k > 0) {
-          if (k == to[i] && len1[k-1]) {
+          if (k == to[i] && count[k-1]) {
             tmp1 = VECTOR_ELT(lookup, k-1);
-            INTEGER(f2__)[thislen] = INTEGER(tmp1)[len1[k-1]-1];
+            INTEGER(f2__)[thislen] = INTEGER(tmp1)[count[k-1]-1];
             ++thislen;
           } else if (k < to[i]) {
             tmp1 = VECTOR_ELT(lookup, k-1);
             tmp2 = VECTOR_ELT(lookup, to[i]-1);
-            j=len1[k-1]-1; m=len1[to[i]-1]-1;
+            j=count[k-1]-1; m=count[to[i]-1]-1;
             while (j>=0 && m>=0) {
               if ( INTEGER(tmp1)[j] == INTEGER(tmp2)[m] ) {
                 INTEGER(f2__)[thislen] = INTEGER(tmp1)[j];
